/*
 * Qt5-UKUI's Library
 *
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Jing Tan <tanjing@kylinos.cn>
 *
 */

import QtQuick 2.6
import QtQuick.Layouts 1.2
import QtQuick.Templates 2.5 as T
import org.ukui.qqc2style.private 1.0 as StylePrivate
import QtQuick.Controls 2.5

T.MenuItem {
    id: controlRoot

    palette: app.palette
    font: app.font


    implicitWidth: Math.max(implicitBackgroundWidth + leftInset + rightInset,
                            image.width + label.width + labelLeftSpace + leftPadding + rightPadding +
                            (controlRoot.checkable ? /*arrow.implicitWidth*/menuItem.imageWidth + menuItem.imageSpace : 0) +
                            (controlRoot.subMenu != null ? /*indicator.implicitWidth*/menuItem.imageWidth + menuItem.imageSpace : 0) + itemLeftSpace)
    implicitHeight: visible ? Math.max(implicitBackgroundHeight + topInset + bottomInset,
                                       Math.max(implicitContentHeight,
                                      (controlRoot.subMenu ? indicator.implicitHeight : 0) + topPadding + bottomPadding), menuItem.normalHeight) : 0

    baselineOffset: contentItem.y + contentItem.baselineOffset

    padding: menuItem.topBottomPadding

    property bool itemHasChecked: controlRoot.checkable && controlRoot.checked

    property int gridUnit: fontMetrics.height
    property int labelLeftSpace: 0
    property int itemLeftSpace: (!(itemHasChecked)/* && !controlRoot.subMenu*/) ? controlRoot.leftPadding : indicatorImage.width + 4 + controlRoot.leftPadding

    property var bacDtColor: !controlRoot.enabled ? menuItem.disableBC :
                                                    controlRoot.pressed ? menuItem.clickedBC :
                                                                            controlRoot.hovered ? menuItem.hoveredBC : menuItem.normalBC

    leftPadding: menuItem.leftRightPadding/* : Math.floor(gridUnit/4)*2*/
    rightPadding: menuItem.leftRightPadding// Math.floor(gridUnit/4)*2
    hoverEnabled: true

    onItemHasCheckedChanged: {
        itemLeftSpace = (!(itemHasChecked)/* && !controlRoot.subMenu*/) ? controlRoot.leftPadding : indicatorImage.width + 4 + controlRoot.leftPadding

    }

    StylePrivate.APPParameter{
        id: appP
    }

    NormalControlColor{
    id:ncColor
    }
    ParseInterface{
        id: parseInterface
    }

    function getStartColor(dtcolor)
    {
        return parseInterface.pStartColor(dtcolor)
    }
    function getEndColor(dtcolor)
    {
        return parseInterface.pEndColor(dtcolor)
    }
    Shortcut {
        //in case of explicit & the button manages it by itself
        enabled: !(RegExp(/\&[^\&]/).test(controlRoot.text))

        onActivated: {
            if (controlRoot.checkable) {
                controlRoot.toggle();
            } else {
                controlRoot.clicked();
            }
        }
    }

    contentItem:Item{
        x: itemLeftSpace + controlRoot.leftPadding
//        anchors.left: controlRoot.left
//        anchors.leftMargin: itemLeftSpace
//        anchors.right: controlRoot.right
//        anchors.rightMargin: controlRoot.rightPadding
        implicitHeight: Math.max(image.height, label.height)
        Image {
            id: image
            source: //controlRoot.icon.source
            {
                if((controlRoot.icon.source.toString() === "" && controlRoot.icon.name.toString() === ""))
                    return controlRoot.icon.source

                var model = !controlRoot.enabled ? "disenable" : appP.isDark ? "highlight" : /*controlRoot.pressed ? "highlight" : controlRoot.hovered ? "highlight" :*/ "normal"

                let s = controlRoot.icon.source.toString()
                return "image://imageProvider/" + s + "/" + model;
            }
            anchors.verticalCenter: parent.verticalCenter
            width: (controlRoot.display !== AbstractButton.TextOnly && source !== "" && status != Image.Null) ? menuItem.imageWidth : 0
            visible: controlRoot.display !== AbstractButton.TextOnly && source !== "" && status != Image.Null
        }
        Label {
            id:label
            text: controlRoot.text
            font:controlRoot.font
            color: getStartColor(controlRoot.enabled ? menuItem.normalTextColor : menuItem.disableTextColor)
            visible: controlRoot.display !== AbstractButton.IconOnly && text !== ""

            anchors.verticalCenter: parent.verticalCenter

            anchors.left: image.right
            anchors.leftMargin: labelLeftSpace//(image.visible && image.status !== Image.Null) ? 4 : labelLeftSpace
        }

    }

    indicator: Image {
        id:indicatorImage
        anchors.left: controlRoot.mirrored ?controlRoot.right : controlRoot.left
        anchors.leftMargin: controlRoot.mirrored ? - controlRoot.rightPadding -  menuItem.imageWidth : controlRoot.leftPadding

        anchors.verticalCenter: controlRoot.verticalCenter

        visible: controlRoot.checked
        source: {
//            return "image://imageProvider/file:///usr/share/icons/ukui-icon-theme-default/scalable/actions/object-select-symbolic.svg/normal"

            var model = "normal"
            if( appP.isDark)
                model = "highlight"
            var icon = "object-select-symbolic";
            if(controlRoot.checkable){
                console.log("app themehasiconnnn", app.themeHasIcon(icon))
                if(!app.themeHasIcon(icon))
                    icon = "file:///usr/share/icons/ukui-icon-theme-default/scalable/actions/object-select-symbolic.svg"
                return "image://imageProvider/" + icon + "/" + model
            }
            else
                return ""
        }
        width: menuItem.imageWidth
        height: menuItem.imageWidth
    }

    arrow: Image {
        id: arrowImage
        anchors.left: controlRoot.mirrored ?controlRoot.left : controlRoot.right
        anchors.leftMargin: controlRoot.mirrored ? controlRoot.leftPadding : - controlRoot.rightPadding -  menuItem.imageWidth
        anchors.verticalCenter: parent.verticalCenter
        width: menuItem.imageWidth
        height: menuItem.imageWidth

        visible: controlRoot.subMenu
        mirror: controlRoot.mirrored
        source: {
            var model = "normal"
            if( appP.isDark)
                model = "highlight"
            var icon = "ukui-end-symbolic";
            if(!app.themeHasIcon(icon))
                icon = "file:///usr/share/icons/ukui-icon-theme-default/scalable/actions/ukui-end-symbolic.svg"
            return "image://imageProvider/" + icon + "/" + model

            return ""
        }
    }

    background:
        Rectangle {
        id: backrect
        width: controlRoot.width
        height: controlRoot.height
        radius: menuItem.radius
        //opacity: (controlRoot.highlighted || controlRoot.hovered || controlRoot.pressed || controlRoot.checked) ? 1 : 0
        border.width: 1
        color: getStartColor(bacDtColor)

        Rectangle{
            id: gradientRec
            anchors.fill: parent
            radius: parent.radius
            border.width: parent.border.width
            border.color: parent.border.color
            gradient:  Gradient {
                id:gradient
                GradientStop { position: 0.0; color: getStartColor(bacDtColor) }

                GradientStop { position: 1.0; color: getEndColor(bacDtColor)}
            }
            visible: !parseInterface.isSolidPattern(bacDtColor)
        }
    }

    onPressedChanged: {
        stateChange()
    }
    onHoveredChanged: {
        stateChange()

    }
    Component.onCompleted: {
        stateChange()
    }
    function stateChange()
    {
        bacDtColor = !controlRoot.enabled ? menuItem.disableBC :
                                                            controlRoot.pressed ? menuItem.clickedBC :
                                                                                    controlRoot.hovered ? menuItem.hoveredBC : menuItem.normalBC

    backrect.border.color = getStartColor(!controlRoot.enabled ? menuItem.disableBorderColor :
                                                  controlRoot.pressed ? menuItem.clickBorderColor :
                                                  controlRoot.hovered ? menuItem.hoverBorderColor : menuItem.normalBorderColor);
    controlRoot.update()
    }
    property variant fontMetrics: TextMetrics {
        text: "M"
        function roundedIconSize(size) {
            if (size < 16) {
                return size;
            } else if (size < 22) {
                return 16;
            } else if (size < 32) {
                return 22;
            } else if (size < 48) {
                return 32;
            } else if (size < 64) {
                return 48;
            } else {
                return size;
            }
        }
    }

    StylePrivate.UKUIMenuItem{
        id: menuItem
    }

    StylePrivate.APPParameter{
        id: app
    }
}





