/*
 * Qt5-UKUI's Library
 *
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Jing Tan <tanjing@kylinos.cn>
 *
 */

import QtQuick 2.0
import QtQuick.Templates 2.5 as T
import QtQuick.Layouts 1.3
import QtQuick.Controls 2.0
import QtGraphicalEffects 1.0
import org.ukui.qqc2style.private 1.0 as StylePrivate



Rectangle {
    id:root

    property T.AbstractButton controlRoot: root.parent
    property int _radius : 6
    property bool useGradient: (controlStartColor !== controlEndColor)
    property color controlStartColor
    property color controlEndColor
    property color borderColor

    color: /*gradientRec.visible ? "transparent" :*/controlStartColor
    opacity: (controlRoot.flat && !controlRoot.pressed && !controlRoot.hovered) ? 0 : 1
    radius: _radius // 可选：设置圆角半径
    border.width: controlRoot.focus ? btn.focusBorderWidth : btn.borderWidth
    border.color: borderColor

    Rectangle {
        id: gradientRec
        anchors.fill: parent
        radius: _radius // 可选：设置圆角半径
        border.width: parent.border.width
        border.color: parent.border.color

        gradient:  Gradient {
            id:gradient
            GradientStop { position: 0.0; color: controlStartColor }

            GradientStop { position: 1.0; color: controlEndColor }
        }
        visible: useGradient
    }

    NormalControlColor{
        id:ncColor
    }

    ParseInterface{
        id: parseInterface
    }
    StylePrivate.UKUIButton {
        id: btn
        onParametryChanged:{
            changeColor(controlRoot.highlighted, controlRoot.enabled, controlRoot.pressed, controlRoot.hovered, controlRoot.focus)
            //root.color = "red"
            root.update();
        }
        //property alias lRMargin: leftRightMargin
    }
    StylePrivate.APPParameter{
        id: app
        onParametryChanged:{
            changeColor(controlRoot.highlighted, controlRoot.enabled, controlRoot.pressed, controlRoot.hovered, controlRoot.focus)
            //root.color = "blue"

            root.update();

        }
    }
    function changeColor(highlighted, enable, click, hover, focus){
        var closeBtn = false;
        var windowBtn = false;
        var importBtn = false;
        if(controlRoot.hasOwnProperty("buttonType")){
            closeBtn = (controlRoot.buttonType === "closeButton");
            windowBtn = (controlRoot.buttonType === "windowButton");
            importBtn = (controlRoot.buttonType === "importButton");
        }

        if(highlighted || importBtn){
            controlStartColor = !enable ? parseInterface.pStartColor(btn.disableHBC) :
                                 click ? parseInterface.pStartColor(btn.clickedHBC) :
                                 hover ? parseInterface.pStartColor(btn.hoveredHBC) :
                                         parseInterface.pStartColor(btn.normalHBC)
            controlEndColor = !enable ? parseInterface.pEndColor(btn.disableHBC) :
                                 click ? parseInterface.pEndColor(btn.clickedHBC) :
                                 hover ? parseInterface.pEndColor(btn.hoveredHBC) :
                                                          parseInterface.pEndColor(btn.normalHBC)

            borderColor = !enable ? parseInterface.pStartColor(btn.disableBorderHColor) :
                                    focus ? parseInterface.pStartColor(btn.focusBorderColor) :
                                    click ? parseInterface.pStartColor(btn.clickBorderHColor) :
                                    hover ? parseInterface.pStartColor(btn.hoverBorderHColor) :
                                            parseInterface.pStartColor(btn.normalBorderHColor)
        }
        else if(closeBtn){
            controlStartColor = !enable ? parseInterface.pStartColor(btn.disableCloseBC) :
                             click ? parseInterface.pStartColor(btn.clickedCloseBC) :
                             hover ? parseInterface.pStartColor(btn.hoveredCloseBC) :
                                     parseInterface.pStartColor(btn.normalCloseBC)

            controlEndColor = !enable ? parseInterface.pEndColor(btn.disableCloseBC) :
                             click ? parseInterface.pEndColor(btn.clickedCloseBC) :
                             hover ? parseInterface.pEndColor(btn.hoveredCloseBC) :
                                     parseInterface.pEndColor(btn.normalCloseBC)

            borderColor = !enable ? parseInterface.pStartColor(btn.disableBorderCloseColor) :
                            click ? parseInterface.pStartColor(btn.clickBorderCloseColor) :
                            focus ? parseInterface.pStartColor(btn.focusBorderColor) :
                            hover ? parseInterface.pStartColor(btn.hoverBorderCloseColor) :
                                    parseInterface.pStartColor(btn.normalBorderCloseColor)
        }
        else if(windowBtn){
            controlStartColor = !enable ? parseInterface.pStartColor(btn.disableWindowBC) :
                             click ? parseInterface.pStartColor(btn.clickedWindowBC) :
                             hover ? parseInterface.pStartColor(btn.hoveredWindowBC) :
                                     parseInterface.pStartColor(btn.normalWindowBC)

            controlEndColor = !enable ? parseInterface.pEndColor(btn.disableWindowBC) :
                             click ? parseInterface.pEndColor(btn.clickedWindowBC) :
                             hover ? parseInterface.pEndColor(btn.hoveredWindowBC) :
                                     parseInterface.pEndColor(btn.normalWindowBC)

            borderColor = !enable ? parseInterface.pStartColor(btn.disableBorderWindowColor) :
                            click ? parseInterface.pStartColor(btn.clickBorderWindowColor) :
                            focus ? parseInterface.pStartColor(btn.focusBorderColor) :
                            hover ? parseInterface.pStartColor(btn.hoverBorderWindowColor) :
                                    parseInterface.pStartColor(btn.normalBorderWindowColor)
        }
        else{
            var disableStartColor       = parseInterface.pStartColor(btn.disableBC);
            var disableEndColor       = parseInterface.pEndColor(btn.disableBC);

            var clickStartColor         = parseInterface.pStartColor(btn.clickedBC);
            var clickEndColor         = parseInterface.pEndColor(btn.clickedBC);

            var normalStartColor        = (controlRoot.flat ? "transparent" : parseInterface.pStartColor(btn.normalBC));
            var normalEndColor        = (controlRoot.flat ? "transparent" : parseInterface.pEndColor(btn.normalBC));

            var hoverStartColor    = parseInterface.pStartColor(btn.hoveredBC);
            var hoverEndColor      = parseInterface.pEndColor(btn.hoveredBC);

            var disableBorderColor = parseInterface.pStartColor(btn.disableBorderColor);
            var focusBorderColor   = parseInterface.pStartColor(btn.focusBorderColor  );
            var clickBorderColor   = parseInterface.pStartColor(btn.clickBorderColor  );
            var hoverBorderColor   = parseInterface.pStartColor(btn.hoverBorderColor  );
            var normalBorderColor  = parseInterface.pStartColor(btn.normalBorderColor );

            if(controlRoot.hasOwnProperty("disableStartColor"))
                disableStartColor = controlRoot.disableStartColor
            if(controlRoot.hasOwnProperty("disableEndColor"))
                disableEndColor = controlRoot.disableEndColor
            if(controlRoot.hasOwnProperty("clickStartColor"))
                clickStartColor = controlRoot.clickStartColor
            if(controlRoot.hasOwnProperty("clickEndColor"))
                clickEndColor = controlRoot.clickEndColor

            if(controlRoot.hasOwnProperty("normalStartColor"))
                normalStartColor = controlRoot.normalStartColor
            if(controlRoot.hasOwnProperty("normalEndColor"))
                normalEndColor = controlRoot.normalEndColor

            if(controlRoot.hasOwnProperty("hoverStartColor"))
                hoverStartColor = controlRoot.hoverStartColor
            if(controlRoot.hasOwnProperty("hoverEndColor"))
                hoverEndColor = controlRoot.hoverEndColor

            if(controlRoot.hasOwnProperty("disableBorderColor"))
                disableBorderColor = controlRoot.disableBorderColor

            if(controlRoot.hasOwnProperty("focusBorderColor"))
                focusBorderColor = controlRoot.focusBorderColor
            if(controlRoot.hasOwnProperty("clickBorderColor"))
                clickBorderColor = controlRoot.clickBorderColor

            if(controlRoot.hasOwnProperty("hoverBorderColor"))
                hoverBorderColor = controlRoot.hoverBorderColor
            if(controlRoot.hasOwnProperty("normalBorderColor"))
                normalBorderColor = controlRoot.normalBorderColor


            controlStartColor = !enable ? disableStartColor :
                             click ? clickStartColor :
                             hover ? hoverStartColor :
                                     normalStartColor

            controlEndColor = !enable ? disableEndColor :
                             click ? clickEndColor :
                             hover ? hoverEndColor :
                                     normalEndColor

            borderColor = !enable ? disableBorderColor :
                            focus ? focusBorderColor :
                            click ? clickBorderColor :
                            hover ? hoverBorderColor :
                                    normalBorderColor
        }

    }


    Component.onCompleted: {
        changeColor(controlRoot.highlighted, controlRoot.enabled, false, false, false);
    }

    Connections {
        target: controlRoot
        function onPressedChanged() {
            changeColor(controlRoot.highlighted, controlRoot.enabled, controlRoot.pressed, controlRoot.hovered, controlRoot.focus)
        }
        function onHoveredChanged() {
            //            if(controlRoot.hovered)
            //                grad.visible = true;
            //            else
            //                grad.visible = false;
            changeColor(controlRoot.highlighted, controlRoot.enabled, false, controlRoot.hovered, controlRoot.focus)
        }
        function onFocusChanged(){
            changeColor(controlRoot.highlighted, controlRoot.enabled, controlRoot.pressed, controlRoot.hovered, controlRoot.focus)
        }
    }
}

