/*
 * Qt5-UKUI's Library
 *
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Jing Tan <tanjing@kylinos.cn>
 *
 */

import QtQuick 2.6
import QtQuick.Templates 2.5 as T
import QtQuick.Layouts 1.3
import QtQuick.Controls 2.0
import org.ukui.qqc2style.private 1.0 as StylePrivate

Item{
    id:root
    property T.AbstractButton controlRoot: root.parent


    property int normalWidth: btn.normalWidth
    property int normalHeight: btn.normalHeight

    property string layout: "center"
    property int leftSpace: 0
    property int labelSpace: 0

//    property var labelColor: controlRoot.enabled ? btn.normalTextColor : btn.disableTextColor

    property bool imageVisible : image.visible || image1.visible
    property var space: {
        if(controlRoot.display === AbstractButton.TextOnly || controlRoot.display === AbstractButton.IconOnly)
            return 0
        return (label.text != "" && imageVisible) ? btn.space : 0
    }

    ParseInterface{
        id: parseInterface
    }
    function getStartColor(dtcolor)
    {
        return parseInterface.pStartColor(dtcolor)
    }
    function getEndColor(dtcolor)
    {
        return parseInterface.pEndColor(dtcolor)
    }

    function adjustWidth(margin){//margin表示边距
        if(controlRoot.display !== AbstractButton.TextUnderIcon){
            var textwidth = (label.text !== "" && controlRoot.display !== AbstractButton.IconOnly) ? label.width : 0
            var imagewidth = (!imageVisible) ? 0 : image.width
            if(controlRoot.display === AbstractButton.IconOnly || label.text === "")
                return imagewidth + margin;
            if(controlRoot.display !== AbstractButton.TextBesideIcon)
                return Math.max(textwidth + margin, imagewidth + margin, normalWidth)
            return Math.max(normalWidth, textwidth + imagewidth + space + margin)
        }
        var textwidth1 = label1.text != "" ? label1.width : 0
        var imagewidth1 = imageVisible ? 0 : image1.width

        return Math.max(normalWidth, textwidth1 + imagewidth1 + space + margin)

    }

    function adjustHeight(margin){//margin表示边距
        if(controlRoot.display !== AbstractButton.TextUnderIcon) {
            var textheight = label1.text != "" ? label1.height : 0
            var imageheight = !imageVisible ? 0 : image1.height

            if(controlRoot.display !== AbstractButton.TextBesideIcon)
                return Math.max(textheight + margin, imageheight + margin, normalHeight)
            return Math.max(normalHeight, textheight + space + margin, imageheight + space + margin)
        }
        var textheight1 = label1.text != "" ? label1.height : 0
        var imageheight1 = !imageVisible ? 0 : image1.height

        var h = Math.max(normalHeight, textheight1 + imageheight1 + space + margin)
        return h
    }

    function labelColor(){
        if(controlRoot.hasOwnProperty("disableLabelColor") && !controlRoot.enabled)
            return controlRoot.disableLabelColor
        else if(controlRoot.hasOwnProperty("clickedLabelColor") && controlRoot.enabled)
            return controlRoot.clickedLabelColor
        else if(controlRoot.hasOwnProperty("hoverLabelColor") && controlRoot.enabled)
            return controlRoot.hoverLabelColor
        else if(controlRoot.hasOwnProperty("normalLabelColor") && controlRoot.enabled)
            return controlRoot.normalLabelColor


            var closeBtn = false;
            var windowBtn = false;
            var importBtn = false;
            if(controlRoot.hasOwnProperty("buttonType")){
                closeBtn = (controlRoot.buttonType === "closeButton");
                windowBtn = (controlRoot.buttonType === "windowButton");
                importBtn = (controlRoot.buttonType === "importButton");
            }

            if(windowBtn){
//                    if(controlRoot.pressed)
//                        labelColor = appP.highlightedTextColor
//                    else if(controlRoot.hovered)
//                        labelColor = appP.brightTextColor
//                    else
                    return (controlRoot.enabled ? btn.normalTextColor : btn.disableTextColor)

            }
            else if(closeBtn &&(controlRoot.pressed || controlRoot.hovered)){
                return btn.highlightTextColor;
            }
            else if(importBtn || controlRoot.highlighted){
                if(controlRoot.flat)
                    return (controlRoot.enabled ? (controlRoot.pressed || controlRoot.hovered) ? btn.highlightTextColor : btn.normalTextColor : btn.disableTextColor)
                else
                    return (controlRoot.enabled ? btn.highlightTextColor : btn.highlightTextDisableColor)

            }
            else{
                return (controlRoot.enabled ? btn.normalTextColor : btn.disableTextColor)
            }

    }

    function adjust()
    {
        root.implicitWidth = adjustWidth(0);
        root.implicitHeight = adjustHeight(0);
        controlRoot.implicitWidth = adjustWidth(2 * btn.leftRightMargin)
        controlRoot.implicitHeight = adjustHeight(2 * btn.upDownMargin)
    }


    StylePrivate.UKUIButton{
        id: btn

    }
    StylePrivate.APPParameter{
        id: appP
        onFontChanged:{
            adjust()
        }
    }
    anchors.centerIn: controlRoot

    Item{
        id:item
        visible: controlRoot.display !== AbstractButton.TextUnderIcon
        implicitWidth: {
            var textwidth = (controlRoot.display !== AbstractButton.IconOnly && label.text !== "") ? label.width : 0
            var imagewidth = imageVisible ? image.width : 0
            if(controlRoot.display === AbstractButton.IconOnly || label.text === "")
                return imagewidth;

            if(controlRoot.display !== AbstractButton.TextBesideIcon)
                return Math.max(textwidth, imagewidth)
            return Math.max( textwidth + imagewidth + space)
        }

        implicitHeight: {
            var textheight = label.visible ? (label.text != "" ? label.height : 0) : 0
            var imageheight = imageVisible ? image.height : 0
            if(controlRoot.display === AbstractButton.IconOnly || label.text === "")
                return imageheight;

            if(controlRoot.display === AbstractButton.TextUnderIcon)
            return Math.max(textheight + imageheight + space, normalHeight);
                return Math.max(textheight, imageheight, normalHeight)

        }
        anchors.left: parent.left
        anchors.leftMargin:
        {
           if(layout == "left"){
               space = leftSpace;
              return leftSpace;
           }
           else
               return (parent.width - width) / 2
        }
        anchors.verticalCenter: parent.verticalCenter


        Image {
            id: image
            source: parseInterface.iconSource(controlRoot)//controlRoot.icon.source
            anchors.verticalCenter: parent.verticalCenter
            width: (controlRoot.display !== AbstractButton.TextOnly && source !== "" && status != Image.Null) ? 16 : 0
            visible: controlRoot.display !== AbstractButton.TextOnly && source !== "" && status != Image.Null

            onSourceChanged: adjust()
        }
        Label {
            id:label
            text: controlRoot.text
            font: controlRoot.font
            color: getStartColor(labelColor())
            visible: controlRoot.display !== AbstractButton.IconOnly && text !== ""

            anchors.verticalCenter: parent.verticalCenter
            verticalAlignment: Text.AlignVCenter

            anchors.left: image.right
            anchors.leftMargin: (image.visible && image.status !== Image.Null) ? labelSpace : 0

            onTextChanged: {
                adjust()
            }
        }

    }
    Item{
        id:item1
        visible: controlRoot.display === AbstractButton.TextUnderIcon
        implicitWidth: {
            var textwidth1 = label1.text != "" ? label1.width : 0
            var imagewidth1 = controlRoot.icon.source === "" ? 0 : image1.width
            if(controlRoot.display === AbstractButton.TextUnderIcon)
            return Math.max(textwidth1, imagewidth1, normalWidth);
            if(controlRoot.display !== AbstractButton.TextBesideIcon)
                return Math.max(textwidth1, imagewidth1, normalWidth)
            return Math.max( textwidth1 + imagewidth1 + space, normalWidth)
        }
        implicitHeight: {
            var textheight1 = label1.text != "" ? label1.height : 0
            var imageheight1 = controlRoot.icon.source === "" ? 0 : image1.height
            if(display === AbstractButton.TextUnderIcon)
            return Math.max(textheight1 + imageheight1 + space, normalHeight);
                return Math.max(textheight1, imageheight1, normalHeight)

        }
        anchors.centerIn: parent
        Image {
            id: image1
            source: parseInterface.iconSource(controlRoot)//controlRoot.icon.source
            anchors.horizontalCenter: parent.horizontalCenter
            width: visible ? 16 : 0
            height: visible ? 16 : 0
            visible: controlRoot.display !== AbstractButton.TextOnly && source !== "" /*&& status != Image.Null*/
            onSourceChanged: adjust()
        }
        Label {
            id:label1
            text: controlRoot.text
            font:controlRoot.font
            color: getStartColor(labelColor())
            visible: controlRoot.display !== AbstractButton.IconOnly && text !== ""

            anchors.horizontalCenter: parent.horizontalCenter
            anchors.top: image1.bottom
            anchors.topMargin: space

            onTextChanged: {
                adjust()
            }
        }

    }
    Component.onCompleted: {
        adjust()
    }

    onVisibleChanged: {
        adjust()
    }
}
